package com.qmth.themis.mq.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.google.gson.Gson;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.dto.WarningDto;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.templete.impl.TaskExamPaperImportTemplete;
import com.qmth.themis.business.templete.impl.TaskExamStudentImportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeExportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeImportTemplete;
import com.qmth.themis.business.threadPool.MyThreadPool;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.util.SimpleBeanUtil;
import com.qmth.themis.mq.service.MqLogicService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * @Description: mq执行逻辑 impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class MqLogicServiceImpl implements MqLogicService {
    private final static Logger log = LoggerFactory.getLogger(MqLogicServiceImpl.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    TMRocketMessageService tmRocketMessageService;

    @Resource
    TEUserLogService teUserLogService;

    @Resource
    TEExamStudentLogService teExamStudentLogService;

    @Resource
    TBSessionService tbSessionService;

    @Resource
    MyThreadPool myThreadPool;

    @Resource
    TOeExamBreakHistoryService tOeExamBreakHistoryService;

    @Resource
    TOeExamRecordService examRecordService;

    @Resource
    TOeFaceVerifyHistoryService faceVerifyHistoryService;

    @Resource
    TOeLivenessVerifyHistoryService livenessVerifyHistoryService;

    @Resource
    CommonService commonService;

    @Resource
    TIeExamInvigilateCallLogService tIeExamInvigilateCallLogService;

    @Resource
    TIeExamInvigilateCallService tIeExamInvigilateCallService;

    @Resource
    TIeInvigilateExceptionInfoService tIeInvigilateExceptionInfoService;

    @Resource
    TEExamService teExamService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    WarningService warningService;

    /**
     * mq最大重试次数逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqMaxReconsumeTime(MqDto mqDto, String key) {
        //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
        Gson gson = new Gson();
        mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 用户轨迹逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqUserLogLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        String tag = mqDto.getTag();
        if (tag.contains(MqTagEnum.USER.name())) {
            teUserLogService.saveUserLogInfo(String.valueOf(mqDto.getBody()), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), mqDto.getObjId());
        } else if (tag.contains(MqTagEnum.STUDENT.name())) {
            if (Objects.nonNull(mqDto.getProperties())) {
                teExamStudentLogService.saveStudentLogInfo(String.valueOf(mqDto.getBody()), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), mqDto.getProperties().get("remark"), mqDto.getObjId(), mqDto.getProperties().get("examStudentId"), mqDto.getProperties().get("examRecordId"));
            } else {
                teExamStudentLogService.saveStudentLogInfo(String.valueOf(mqDto.getBody()), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), mqDto.getObjId());
            }
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * session逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqSessionLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        tbSessionService.saveSessionInfo(JacksonUtil.readJson(JacksonUtil.parseJson(mqDto.getBody()), TBSession.class), mqDto.getTimestamp());
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * task逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqTaskLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> map = (Map<String, Object>) mqDto.getBody();
        String tag = mqDto.getTag();
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            if (tag.contains("Import".toUpperCase())) {
                TaskImportTemplete taskImportTemplete = null;
                if (tag.contains(MqTagEnum.EXAM_STUDENT_IMPORT.name())) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskExamStudentImportTemplete.class);
                } else if (tag.contains(MqTagEnum.ROOM_CODE_IMPORT.name())) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskRoomCodeImportTemplete.class);
                } else if (tag.contains(MqTagEnum.EXAM_PAPER_IMPORT.name())) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskExamPaperImportTemplete.class);
                }
                try {
                    taskImportTemplete.importTask(map);
                } catch (IOException e) {
                    log.error("请求出错", e);
                }
            } else {
                TaskExportTemplete taskExportTemplete = null;
                if (tag.contains(MqTagEnum.ROOM_CODE_EXPORT.name())) {
                    taskExportTemplete = SpringContextHolder.getBean(TaskRoomCodeExportTemplete.class);
                }
                try {
                    taskExportTemplete.exportTask(map);
                } catch (IOException e) {
                    log.error("请求出错", e);
                }
            }
        });
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        mqDto.setBody(JacksonUtil.parseJson(mqDto.getBody()));
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * websocket超时退出逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqWebsocketUnNormalLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);//表示成功处理
        Map<String, Object> tranMap = mqDto.getProperties();
        Long recordId = Long.parseLong(String.valueOf(tranMap.get("recordId")));
        Date clientLastSyncTime = ExamRecordCacheUtil.getClientLastSyncTime(recordId);
        Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        if (Objects.nonNull(objectMap)) {
            Integer diff = 0;
            if (Objects.nonNull(clientLastSyncTime)) {//大于等于当前时间，说明未重连或重登录
                Long l = (System.currentTimeMillis() - clientLastSyncTime.getTime()) / 1000;
                if (l >= 2 && !Objects.equals(ExamRecordCacheUtil.getStatus(recordId), ExamRecordStatusEnum.FINISHED)) {
                    diff = l.intValue();
                    //是否增加断点记录或者交卷处理统一放登录逻辑里处理，这里只更新考试记录状态为中断
                    ExamRecordCacheUtil.setStatus(recordId, ExamRecordStatusEnum.BREAK_OFF);
                }
            }
            //增加异常日志
            Long examId = ExamRecordCacheUtil.getExamId(recordId);
            Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
            Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
            TIeInvigilateExceptionInfo tIeInvigilateExceptionInfo = new TIeInvigilateExceptionInfo(examId, examActivityId, recordId, examStudentId, ExceptionEnum.NET_TIME_OUT.getCode(), ExceptionEnum.NET_TIME_OUT, diff);
            tIeInvigilateExceptionInfoService.saveOrUpdate(tIeInvigilateExceptionInfo);
        }
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        Map map = new HashMap();
        map.put(SystemConstant.MQDTO_OBJ, JacksonUtil.parseJson(mqDto));
        tmRocketMessage.setProp(JacksonUtil.parseJson(map));
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 计算客观分逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqCalculateObjectiveScoreLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        examRecordService.calculateObjectiveScore(param);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 人脸验证保存
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqFaceVerifySaveLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long id = (Long) param.get("id");
        Long recordId = (Long) param.get("recordId");
        String type = (String) param.get("type");
        String photoUrl = (String) param.get("photoUrl");
        Integer faceCount = (Integer) param.get("faceCount");
        Double similarity = (Double) param.get("similarity");
        Double realness = (Double) param.get("realness");
        Long time = (Long) param.get("time");
        String exception = (String) param.get("exception");
        faceVerifyHistoryService.save(id, recordId, type, photoUrl, faceCount, similarity, realness, time, exception);
        VerifyExceptionEnum warningEnum = VerifyExceptionEnum.valueOf(exception);
        WarningDto warningDto = new WarningDto(warningEnum, faceCount, realness, recordId, photoUrl);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        if (Objects.equals(VerifyExceptionEnum.NONE, warningEnum)) {//无异常，往考生日志表里插一条
            JSONObject jsonObject = new JSONObject();
            jsonObject.put(PhotoTypeEnum.FACE_VERIFY_PHOTO.name(), photoUrl);
            TEExamStudentLog teExamStudentLog = new TEExamStudentLog(type, ExamTypeEnum.valueOf(type).getCode(), jsonObject.toJSONString(), examStudentCacheBean.getStudentId(), examStudentCacheBean.getId(), recordId);
            teExamStudentLogService.saveOrUpdate(teExamStudentLog);
        } else {
            if (Objects.equals(VerifyExceptionEnum.FACE_COUNT_ERROR, warningEnum)) {//人脸数量异常
                warningService.faceCountError(warningDto);
            } else if (Objects.equals(VerifyExceptionEnum.FACE_COMPARE_ERROR, warningEnum)) {//人脸比对异常
                warningService.faceCompareError(warningDto);
            } else {
                if (realness.intValue() == 0) {//真实性异常
                    warningService.realnessError(warningDto);
                }
            }
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 活体验证
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqLivenessVerifySaveLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long id = (Long) param.get("id");
        Long recordId = (Long) param.get("recordId");
        String type = (String) param.get("type");
        String actions = (String) param.get("actions");
        Integer retry = (Integer) param.get("retry");
        Long startTime = (Long) param.get("startTime");
        Long finishTime = (Long) param.get("finishTime");
        String exception = (String) param.get("exception");
        livenessVerifyHistoryService.save(id, recordId, type, actions, retry, startTime, finishTime, exception);
        VerifyExceptionEnum warningEnum = VerifyExceptionEnum.valueOf(exception);
        JSONObject jsonObject = JSONObject.parseObject(actions);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        if (Objects.equals(VerifyExceptionEnum.NONE, warningEnum)) {//无异常，往考生日志表里插一条
            JSONObject object = new JSONObject();
            object.put(PhotoTypeEnum.LIVENESS_VERIFY_PHOTO.name(), String.valueOf(jsonObject.get("photoUrl")));
            TEExamStudentLog teExamStudentLog = new TEExamStudentLog(type, LivenessTypeEnum.valueOf(type).getTitle(), object.toJSONString(), examStudentCacheBean.getStudentId(), examStudentCacheBean.getId(), recordId);
            teExamStudentLogService.saveOrUpdate(teExamStudentLog);
        } else {
            if (Objects.equals(VerifyExceptionEnum.FACE_COUNT_ERROR, warningEnum)) {//人脸数量异常
                warningService.faceCountError(new WarningDto(warningEnum, Integer.parseInt(String.valueOf(jsonObject.get("faceCount"))), null, recordId, String.valueOf(jsonObject.get("photoUrl"))));
            } else if (Objects.equals(VerifyExceptionEnum.FACE_COMPARE_ERROR, warningEnum)) {//人脸比对异常
                warningService.faceCompareError(new WarningDto(warningEnum, null, null, recordId, String.valueOf(jsonObject.get("photoUrl"))));
            }
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 考试记录数据持久化
     *
     * @param mqDto
     * @param key
     */
    @Override
    public void execMqExamRecordPersistedLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long recordId = (Long) param.get("recordId");
        commonService.persisted(recordId);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    @Override
    @Transactional
    public void execMqRecordUpdateLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long recordId = Long.parseLong(String.valueOf(param.get("recordId")));
        String colName = String.valueOf(param.get("colName"));
        Object colValue = param.get("colValue");
        Integer isDate = Integer.parseInt(String.valueOf(param.get("isDate")));
        if (colName.contains("client_last_sync_time") || colName.contains("last_break_time") || colName.contains("last_start_time")) {
            Date date = new Date();
            date.setTime(Long.parseLong(String.valueOf(colValue)));
            examRecordService.dataUpdate(recordId, colName, date, isDate);
        } else {
            examRecordService.dataUpdate(recordId, colName, colValue, isDate);
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    @Override
    @Transactional
    public void execMqRecordInitLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        examRecordService.dataInit(param);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 监控通话申请逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqLogLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        String tag = mqDto.getTag();
        if (tag.contains(MqTagEnum.MONITOR_LOG.name())) {//监考日志
            TIeExamInvigilateCallLog tIeExamInvigilateCallLog = JacksonUtil.readJson(JacksonUtil.parseJson(mqDto.getBody()), TIeExamInvigilateCallLog.class);
            Long recordId = tIeExamInvigilateCallLog.getExamRecordId();
            Long examId = null, examActivityId = null, examStudentId = null;
            if (Objects.nonNull(ExamRecordCacheUtil.getExamActivityId(recordId))) {
                examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
                examId = ExamRecordCacheUtil.getExamId(recordId);
                examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
            } else {
                TOeExamRecord tOeExamRecord = examRecordService.getById(tIeExamInvigilateCallLog.getExamRecordId());
                examActivityId = tOeExamRecord.getExamActivityId();
                examId = tOeExamRecord.getExamId();
                examStudentId = tOeExamRecord.getExamStudentId();
            }
            tIeExamInvigilateCallLog.setExamId(examId);
            tIeExamInvigilateCallLog.setExamActivityId(examActivityId);
            tIeExamInvigilateCallLog.setExamStudentId(examStudentId);
            QueryWrapper<TIeExamInvigilateCall> tIeExamInvigilateCallQueryWrapper = new QueryWrapper<>();
            tIeExamInvigilateCallQueryWrapper.lambda().eq(TIeExamInvigilateCall::getExamRecordId, tIeExamInvigilateCallLog.getExamRecordId()).eq(TIeExamInvigilateCall::getSource, tIeExamInvigilateCallLog.getSource());
            TIeExamInvigilateCall tIeExamInvigilateCall = tIeExamInvigilateCallService.getOne(tIeExamInvigilateCallQueryWrapper);
            if (Objects.isNull(tIeExamInvigilateCall)) {
                tIeExamInvigilateCall = gson.fromJson(gson.toJson(tIeExamInvigilateCallLog), TIeExamInvigilateCall.class);
                tIeExamInvigilateCall.setId(Constants.idGen.next());
            } else {
                tIeExamInvigilateCall.setLiveUrl(tIeExamInvigilateCallLog.getLiveUrl());
                tIeExamInvigilateCall.setStatus(tIeExamInvigilateCallLog.getStatus());
                tIeExamInvigilateCall.setCallStatus(tIeExamInvigilateCallLog.getCallStatus());
            }
            if (Objects.nonNull(tIeExamInvigilateCall.getUpdateTime())) {
                long dbtimestamp = tIeExamInvigilateCall.getUpdateTime().getTime();
                if (mqDto.getTimestamp() > dbtimestamp) {
                    tIeExamInvigilateCallService.updateById(tIeExamInvigilateCall);
                }
            } else {
                tIeExamInvigilateCallService.saveOrUpdate(tIeExamInvigilateCall);
            }
            tIeExamInvigilateCallLogService.saveOrUpdate(tIeExamInvigilateCallLog);
        } else if (tag.contains(MqTagEnum.EXCEPTION_LOG.name())) {//考试断点异常日志
            JSONObject jsonObject = JSONObject.parseObject(String.valueOf(mqDto.getBody()));
            ExceptionEnum exceptionEnum = ExceptionEnum.valueOf(ExceptionEnum.convertToName(String.valueOf(jsonObject.getJSONObject("reason").get("type"))));
            Long recordId = Long.parseLong(mqDto.getObjId());
            //获取最近同步时间
            Date clientLastSyncTime = ExamRecordCacheUtil.getClientLastSyncTime(recordId);
            Integer diff = 0;
            if (Objects.nonNull(clientLastSyncTime)) {
                Long l = ((System.currentTimeMillis() - clientLastSyncTime.getTime()) / 1000);
                diff = l.intValue();
            }
            Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
            ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);

            //先查询之前的断点记录
            QueryWrapper<TOeExamBreakHistory> tOeExamBreakHistoryQueryWrapper = new QueryWrapper<>();
            tOeExamBreakHistoryQueryWrapper.lambda().eq(TOeExamBreakHistory::getExamRecordId, recordId);
            List<TOeExamBreakHistory> tOeExamBreakHistoryList = tOeExamBreakHistoryService.list(tOeExamBreakHistoryQueryWrapper);
            //删除历史断点缓存
            if (Objects.nonNull(tOeExamBreakHistoryList) && tOeExamBreakHistoryList.size() > 0) {
                tOeExamBreakHistoryList.forEach(s -> {
                    redisUtil.delete(RedisKeyHelper.examBreakCacheKey(s.getId()));
                });
            }
            //增加断点记录
            TOeExamBreakHistory tOeExamBreakHistory = new TOeExamBreakHistory(ExamRecordCacheUtil.getLastBreakId(recordId), recordId, new Date(), exceptionEnum, exceptionEnum.name());
            tOeExamBreakHistoryService.save(tOeExamBreakHistory);
            redisUtil.setForHash(RedisKeyHelper.examBreakCacheKey(tOeExamBreakHistory.getId()), SimpleBeanUtil.objectToMap(tOeExamBreakHistory));
            teExamStudentLogService.saveStudentLogInfo(SystemOperationEnum.RESUME_PREPARE.name(), SystemOperationEnum.RESUME_PREPARE.getCode(), SystemOperationEnum.RESUME_PREPARE.getCode(), examStudentCacheBean.getStudentId(), examStudentId, recordId);

            //增加异常日志
            Long examId = ExamRecordCacheUtil.getExamId(recordId);
            Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
            TIeInvigilateExceptionInfo tIeInvigilateExceptionInfo = new TIeInvigilateExceptionInfo(examId, examActivityId, recordId, examStudentId, String.valueOf(jsonObject.getJSONObject("reason").get("reason")), exceptionEnum, diff);
            tIeInvigilateExceptionInfoService.saveOrUpdate(tIeInvigilateExceptionInfo);
        } else if (tag.contains(MqTagEnum.WARNING_LOG.name())) {//考试预警日志

        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 重新算分
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqCalculateScoreLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long examId = (Long) param.get("examId");
        Long taskId = (Long) param.get("taskId");
        teExamService.calculateScore(examId, taskId);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }
}
